#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Api::MeasuresController < Api::RestController

  before_filter :init_project

  require_role Role::ADMIN, :only => [ :create, :update, :destroy ]

  def index
    sql = 'status=:status '
    values = {:status => Snapshot::STATUS_PROCESSED}

    #--------
    # STEP 1 : load snapshots and projects
    #--------

    # filter on project
    if @project
      scope=params[:scope] || Project::SCOPE_PROJECT
      if scope == Project::SCOPE_PROJECT
        sql = "#{sql} AND snapshots.project_id=:project_id"
        values[:project_id]=@project.id
      else
        sql = "#{sql} AND snapshots.root_snapshot_id=:root_snapshot_id AND snapshots.scope=:scope"
        values[:root_snapshot_id]=@project.last_snapshot.id
        values[:scope]=params[:scope]
      end
    else
      sql="#{sql} AND snapshots.scope=:scope AND snapshots.root_snapshot_id IS NULL AND projects.enabled=:enabled"
      values[:scope]=Project::SCOPE_PROJECT
      values[:enabled]=true
    end

    # timemachine
    if params[:sid] and params[:sid].length>0
      sql = "#{sql} AND snapshots.id in (:sids) "
      values[:sids]=params[:sid].split(',')
    else
      sql = "#{sql} AND islast=:islast "
      values[:islast]=true
    end

    snapshots = Snapshot.find(:all, :include => 'project', :conditions => [sql,values])
    snapshots_by_project = {}
    snapshots.each do |snapshot|
      snapshots_by_project[snapshot.project]||=[]
      snapshots_by_project[snapshot.project]<<snapshot
    end

    #--------
    # STEP 2 : load measures and parameters
    #--------
    sql = 'rule_id IS NULL AND rules_category_id IS NULL AND project_measures.snapshot_id IN (:sids)'
    values = {:sids => snapshots.collect{|s| s.id}}

    # filter metrics
    if params[:metrics]
      sql = "#{sql} AND project_measures.metric_id IN (:metrics) "
      values[:metrics]=Metric.ids_from_keys(params[:metrics].split(','))
    end

    measures = ProjectMeasure.find(:all, :include => :measure_parameters, :conditions => [sql,values])
    measures_by_snapshot_id = {}
    measures.each do |measure|
      measures_by_snapshot_id[measure.snapshot_id]||=[]
      measures_by_snapshot_id[measure.snapshot_id]<<measure
    end

    #--------
    # STEP 3 : load asynchronous measures and parameters
    #--------
    sql = 'async_measure_snapshots.snapshot_id IN (:sids)'
    values = {:sids => snapshots.collect{|s| s.id}}

    # filter project
    if @project
      sql = "#{sql} AND async_measure_snapshots.project_id=:project_id"
      values[:project_id]=@project.id
    end

    # filter metrics
    if params[:metrics]
      sql = "#{sql} AND async_measure_snapshots.metric_id IN (:metrics) "
      values[:metrics]=Metric.ids_from_keys(params[:metrics].split(','))
    end

    async_measures = AsyncMeasureSnapshot.find(:all, :include => {:async_measure => :measure_parameters}, :conditions => [sql,values])
    async_measures.each do |async_measure|
      measures_by_snapshot_id[async_measure.snapshot_id]||=[]
      measures_by_snapshot_id[async_measure.snapshot_id]<<async_measure.measure
    end

    #--------
    # STEP 4 : format response
    #--------
    respond_to do |format|
      format.json{ render :json => rest_to_json(snapshots_by_project, measures_by_snapshot_id) }
      format.xml { render :xml => rest_to_xml(snapshots_by_project, measures_by_snapshot_id) }
    end

  end


  def create
    measure = ProjectMeasure.new
    measure.value=params[:value]
    measure.text_value=params[:text_value]
    if params[:date]
      measure.measure_date=params[:date]
    else
      measure.measure_date=Time.now
    end
    measure.metric=Metric.by_key(params[:metric])
    measure.snapshot=nil
    measure.project=@project
    measure.save!

    java_facade.registerAsyncMeasure(measure.id.to_i)
    
    rest_status_ok

  rescue
    rest_status_ko(measure.errors.full_messages.join("."), 400)
  end


  protected

  def init_project
    project_key = params[:project_id]
    if project_key
      if project_key.to_i>0
        @project = Project.find(:first, :conditions => {:id => project_key.to_i, :enabled => true})
      else
        @project = Project.find(:first, :conditions => {:kee => project_key, :enabled => true})
      end
      if @project.nil?
        rest_status_ko("Project [#{project_key}] not found", 404)
        return
      end
    end
  end

  def rest_to_json(snapshots_by_project, measures_by_snapshot_id)
    result=[]
    snapshots_by_project.each_pair do |project,snapshots|
      result<<project.to_hash_json(params, snapshots, measures_by_snapshot_id)
    end
    json = result.to_json
    json = params['callback'] + '(' + json + ');' if params['callback']
    json
  end

  def rest_to_xml(snapshots_by_project, measures_by_snapshot_id)
    xml = Builder::XmlMarkup.new(:indent => 0)
    xml.instruct!

    xml.projects do
      snapshots_by_project.each_pair do |project,snapshots|
        xml << project.to_xml(params, snapshots, measures_by_snapshot_id)
      end
    end
  end

end
